@@ -108,12 +108,12 @@ intern void pth_key_destroydata(pth_t t)
                 destructor = NULL;
                 data = NULL;
                 if (pth_keytab[key].used) {
-                    if (t->data_value[key]) {
+                    if (t->data_value[key] != NULL) {
                         data = (void *)t->data_value[key];
                         t->data_value[key] = NULL;
                         t->data_count--;
                         destructor = pth_keytab[key].destructor;
-                     }
+                    }
                 }
                 if (destructor != NULL)
                     destructor(data);
