@@ -100,11 +100,17 @@ int pth_init(void)
 }
 
 /* kill the package internals */
-void pth_kill(void)
+int pth_kill(void)
 {
+    if (pth_current != pth_main) 
+        return_errno(FALSE, EINVAL);
+    pth_debug1("pth_kill: enter");
     pth_scheduler_kill();
     pth_initialized = FALSE;
-    return;
+    pth_tcb_free(pth_sched);
+    pth_tcb_free(pth_main);
+    pth_debug1("pth_kill: leave");
+    return TRUE;
 }
 
 /* scheduler control/query */
