@@ -197,7 +197,7 @@ intern int pth_mctx_set(
 #if !defined(SS_DISABLE) && defined(SA_DISABLE)
 #define SS_DISABLE SA_DISABLE
 #endif
-#if !defined(HAVE_SS_SP) && defined(HAVE_SS_BASE)
+#if PTH_MCTX_STK(sas) && !defined(HAVE_SS_SP) && defined(HAVE_SS_BASE)
 #define ss_sp ss_base
 #endif
 
