@@ -33,6 +33,9 @@
 
 #if PTH_SYSCALL_HARD
 #include <sys/syscall.h>
+#ifdef HAVE_SYS_SOCKETCALL_H
+#include <sys/socketcall.h>
+#endif
 #define pth_sc(func) PTH_SC_##func
 #else
 #define pth_sc(func) func
@@ -110,16 +113,43 @@ pid_t waitpid(pid_t wpid, int *status, int options)
 }
 #endif
 
+#if defined(SYS_socketcall) && defined(SOCKOP_connect) && defined(SOCKOP_accept) /* mainly Linux */
+intern int _pth_socketcall(int call, ...)
+{
+    va_list ap;
+    unsigned long args[3];
+
+    va_start(ap, call);
+    switch (call) {
+        case SOCKOP_connect:
+            args[0] = (unsigned long)va_arg(ap, int);
+            args[1] = (unsigned long)va_arg(ap, struct sockaddr *);
+            args[2] = (unsigned long)va_arg(ap, int);
+            break;
+        case SOCKOP_accept:
+            args[0] = (unsigned long)va_arg(ap, int);
+            args[1] = (unsigned long)va_arg(ap, struct sockaddr *);
+            args[2] = (unsigned long)va_arg(ap, int *);
+            break;
+    }
+    va_end(ap);
+    return syscall(SYS_socketcall, call, args);
+}
+#endif
+
 /* Pth hard wrapper for syscall connect(2) */
 #if cpp
 #if defined(SYS_connect)
 #define PTH_SC_connect(a1,a2,a3) ((int)syscall(SYS_connect,(a1),(a2),(a3)))
+#elif defined(SYS_socketcall) && defined(SOCKOP_connect) /* mainly Linux */
+#define PTH_SC_connect(a1,a2,a3) ((int)_pth_socketcall(SOCKOP_connect,(a1),(a2),(a3)))
 #else
 #define PTH_SC_connect connect
 #endif
 #endif /* cpp */
-#if PTH_SYSCALL_HARD && defined(SYS_connect)
-int connect(int s, const struct sockaddr *addr, int addrlen)
+#if PTH_SYSCALL_HARD &&\
+    (defined(SYS_connect) || (defined(SYS_socketcall) && defined(SOCKOP_connect)))
+int connect(int s, const struct sockaddr *addr, socklen_t addrlen)
 {
     pth_implicit_init();
     return pth_connect(s, addr, addrlen);
@@ -130,12 +160,15 @@ int connect(int s, const struct sockaddr *addr, int addrlen)
 #if cpp
 #if defined(SYS_accept)
 #define PTH_SC_accept(a1,a2,a3) ((int)syscall(SYS_accept,(a1),(a2),(a3)))
+#elif defined(SYS_socketcall) && defined(SOCKOP_accept) /* mainly Linux */
+#define PTH_SC_accept(a1,a2,a3) ((int)_pth_socketcall(SOCKOP_accept,(a1),(a2),(a3)))
 #else
 #define PTH_SC_accept accept
 #endif
 #endif /* cpp */
-#if PTH_SYSCALL_HARD && defined(SYS_accept)
-int accept(int s, struct sockaddr *addr, int *addrlen)
+#if PTH_SYSCALL_HARD &&\
+    (defined(SYS_accept) || (defined(SYS_socketcall) && defined(SOCKOP_accept)))
+int accept(int s, struct sockaddr *addr, socklen_t *addrlen)
 {
     pth_implicit_init();
     return pth_accept(s, addr, addrlen);
@@ -144,13 +177,15 @@ int accept(int s, struct sockaddr *addr, int *addrlen)
 
 /* Pth hard wrapper for syscall select(2) */
 #if cpp
-#if defined(SYS_select)
+#if defined(SYS__newselect) /* mainly Linux */
+#define PTH_SC_select(a1,a2,a3,a4,a5) ((int)syscall(SYS__newselect,(a1),(a2),(a3),(a4),(a5)))
+#elif defined(SYS_select)
 #define PTH_SC_select(a1,a2,a3,a4,a5) ((int)syscall(SYS_select,(a1),(a2),(a3),(a4),(a5)))
 #else
 #define PTH_SC_select select
 #endif
 #endif /* cpp */
-#if PTH_SYSCALL_HARD && defined(SYS_select)
+#if PTH_SYSCALL_HARD && (defined(SYS__newselect) || defined(SYS_select))
 int select(int nfds, fd_set *readfds, fd_set *writefds, 
            fd_set *exceptfds, struct timeval *timeout)
 {
