@@ -111,7 +111,7 @@ intern pth_t pth_tcb_alloc(unsigned int stacksize, void *stackaddr)
             t->stack = (char *)(stackaddr);
         else {
             if ((t->stack = (char *)malloc(stacksize)) == NULL) {
-                free(t);
+                errno_shield { free(t); }
                 return NULL;
             }
         }
