@@ -984,6 +984,8 @@ int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex,
     );
     if (!pth_cond_await((pth_cond_t *)(*cond), (pth_mutex_t *)(*mutex), ev))
         return errno;
+    if (pth_event_occurred(ev))
+        return ETIMEDOUT;
     return OK;
 }
 
