@@ -40,16 +40,29 @@ struct query {
 };
 
 /* our worker thread which translates the string to upper case */
+typedef struct {
+    pth_msgport_t mp;
+    pth_event_t ev;
+} worker_cleanup_t;
+static void worker_cleanup(void *arg)
+{
+    worker_cleanup_t *wc = (worker_cleanup_t *)arg;
+    pth_event_free(wc->ev, PTH_FREE_THIS);
+    pth_msgport_destroy(wc->mp);
+    return;
+}
 static void *worker(void *_dummy)
 {
+    worker_cleanup_t wc;
     pth_msgport_t mp;
     pth_event_t ev;
     struct query *q;
     int i;
 
     fprintf(stderr, "worker: start\n");
-    mp = pth_msgport_create("worker");
-    ev = pth_event(PTH_EVENT_MSG, mp);
+    wc.mp = mp = pth_msgport_create("worker");
+    wc.ev = ev = pth_event(PTH_EVENT_MSG, mp);
+    pth_cleanup_push(worker_cleanup, &wc);
     for (;;) {
          if ((i = pth_wait(ev)) != 1)
              continue;
@@ -61,8 +74,6 @@ static void *worker(void *_dummy)
              pth_msgport_reply((pth_message_t *)q);
          }
     }
-    pth_event_free(ev, PTH_FREE_ALL);
-    pth_msgport_destroy(mp);
     return NULL;
 }
 
@@ -111,7 +122,7 @@ int main(int argc, char *argv[])
 
     t_attr = pth_attr_new();
     pth_attr_set(t_attr, PTH_ATTR_NAME, "worker");
-    pth_attr_set(t_attr, PTH_ATTR_JOINABLE, FALSE);
+    pth_attr_set(t_attr, PTH_ATTR_JOINABLE, TRUE);
     pth_attr_set(t_attr, PTH_ATTR_STACK_SIZE, 16*1024);
     t_worker = pth_spawn(t_attr, worker, NULL);
     pth_attr_set(t_attr, PTH_ATTR_NAME, "ticker");
@@ -157,9 +168,14 @@ int main(int argc, char *argv[])
         fprintf(stderr, "main: in <-- <%s>\n", q->string);
     }
 
+    free(q);
     pth_event_free(ev, PTH_FREE_THIS);
     pth_event_free(evt, PTH_FREE_THIS);
     pth_msgport_destroy(mp);
+    pth_cancel(t_worker);
+    pth_join(t_worker, NULL);
+    pth_cancel(t_ticker);
+    pth_join(t_ticker, NULL);
     pth_kill();
     return 0;
 }
