@@ -53,6 +53,8 @@ static void *inthandler(void *_arg)
          pth_sigwait(&sigs, &sig);
          fprintf(stderr, "inthandler: SIGINT received (#%d)\n", n);
     }
+
+    fprintf(stderr, "inthandler: cancelling child1 and child2\n");
     pth_cancel(child1);
     pth_cancel(child2);
 
@@ -62,7 +64,7 @@ static void *inthandler(void *_arg)
 
 static void child_cleanup(void *arg)
 {
-    fprintf(stderr, "Child %s cleanup\n", (char *)arg);
+    fprintf(stderr, "%s: running cleanup\n", (char *)arg);
     return;
 }
 
@@ -139,7 +141,7 @@ int main(int argc, char *argv[])
     pth_attr_destroy(attr);
 
     /* wait until childs are finished */
-    while (pth_join(NULL, NULL) == 0) ;
+    while (pth_join(NULL, NULL));
 
     fprintf(stderr, "main: exit\n");
     pth_kill();
